from nltk_classifiers import *
import glob
import string
import nltk

POSITIVE_WORDS_FILE = '../../../../Resources/Opinion Mining/Lexicon/positive-words.txt'
NEGATIVE_WORDS_FILE = '../../../../Resources/Opinion Mining/Lexicon/negative-words.txt'

DATA_DIR = '../Data/'
RAW_DATA_DIR = '../Data/Implicit Aspects Annotated/'
PARSED_DATA_DIR = '../Data/ParsedData/'

CONCATENATED_DATA_FILE = "all-data"
TRAINING_DATA_FILE = "training-data"
TEST_DATA_FILE = "test-data"
NON_IMPLICIT_DATA_FILE = "non-implicit"
IMPLICIT_DATA_FILE = "implicit"

TrainPrc = 0.8
TestPrc = 0.2
MustThreeshold = 0.7

def MakeGenericWordList(FileName, List):
    f=open(FileName,'r')
    word = f.readline()
    while word != '':
        word = word.rstrip('\n').rstrip('\r')
        if not word in List:
            List.append(word)
        word = f.readline()    
    return

def ConcatenateData():
    DocList = glob.glob(RAW_DATA_DIR + '*')
    AllDoc = open(DATA_DIR + CONCATENATED_DATA_FILE, 'w')
    for doc in DocList:
        f = open(doc, 'r')
        txt = f.readline()
        while txt != '':
            AllDoc.write(txt)
            txt = f.readline()
        f.close()
    AllDoc.close()
    return

def BuildTrainTestData():
    TrainSize = 0
    TestSize = 0
    FileSize = 0
    
    TrainDoc = open(DATA_DIR + TRAINING_DATA_FILE, 'w')
    TestDoc = open(DATA_DIR + TEST_DATA_FILE, 'w')
    
    f = open(DATA_DIR + CONCATENATED_DATA_FILE, 'r')
    FileSize = FileSize + len(f.read())
    f.close()
    
    f = open(DATA_DIR + CONCATENATED_DATA_FILE, 'r')
    txt = f.readline()

    while txt != '':
        if TrainSize < (FileSize * TrainPrc):
            TrainDoc.write(txt)
            TrainSize = TrainSize + len(txt)
        elif TestSize < (FileSize * TestPrc):
            TestDoc.write(txt)
            TestSize = TestSize + len(txt)
        else:
            TrainDoc.write(txt)
            TrainSize = TrainSize + len(txt)
        txt = f.readline()
    
    f.close()
    
    return

def DataParser():
    NonImplicitDoc = open(PARSED_DATA_DIR + NON_IMPLICIT_DATA_FILE, 'w')
    ImplicitDoc = open(PARSED_DATA_DIR + IMPLICIT_DATA_FILE, 'w')

    f = open(DATA_DIR + TRAINING_DATA_FILE, 'r')
    txt = f.readline()
    while txt != '':
        if '[t]' in txt:
            txt = f.readline()
            continue
        split = txt.split('##') 
        if split[0] == '':
            NonImplicitDoc.write(split[1])
        else:
            ImplicitDoc.write(split[1])
        txt = f.readline()
    
    f.close()
    
    NonImplicitDoc.close()
    ImplicitDoc.close()
    return

def StatisticOccurrence(MustWordList):
    
    WordDic = { }
    IAIWordDic = { }
    Doc = open(DATA_DIR + TEST_DATA_FILE)
        
    txt = Doc.readline()
    while txt != '':
        if '[t]' in txt:
            txt = Doc.readline()
            continue
        
        IAI = []
        split = txt.split('##')
        sentence = split[1]
        if split[0] != '':
            for iai_aspect in split[0].split(','):
                IAI.append(iai_aspect.split('[')[0])
                
        for word in sentence.split():
            if WordDic.has_key(word):
                if word in IAI:
                    if IAIWordDic.has_key(word):
                        IAIWordDic[word] += 1
                    else:
                        IAIWordDic[word] = 1
                WordDic[word] += 1
            else:
                if word in IAI:
                    if IAIWordDic.has_key(word):
                        IAIWordDic[word] += 1
                    else:
                        IAIWordDic[word] = 1
                WordDic[word] = 1
        
        txt = Doc.readline()
                        
    for word, value in IAIWordDic.iteritems():
        if float(value) / float(WordDic[word]) >= MustThreeshold:
            MustWordList.append(word)
            
    return

if __name__ == '__main__':
    #build classifier if it does not exist. Otherwise load the pickle file
    texts = {}
    texts['imp'] = PARSED_DATA_DIR + IMPLICIT_DATA_FILE
    texts['non'] = PARSED_DATA_DIR + NON_IMPLICIT_DATA_FILE
    if not os.path.exists('classifier.pickle'):
        ConcatenateData()
        BuildTrainTestData()
        DataParser()
        train_set = get_train_set(texts)
        classifier = NaiveBayesClassifier.train(train_set)
        pickle.dump(classifier, open('classifier.pickle','w'))
    else: 
        classifier = pickle.load(open('classifier.pickle','r'))
    
    #Get Statistical ocurrence
    MustWord = []
    StatisticOccurrence(MustWord)
    
    PosWordList = []
    NegWordList = []
    
    MakeGenericWordList(POSITIVE_WORDS_FILE, PosWordList)
    MakeGenericWordList(NEGATIVE_WORDS_FILE, NegWordList)
    
    #Make the test and get Precision, Recall and F1
    f_p = 0
    f_n = 0
    v_p = 0
    v_n = 0
    
    f_p_c = 0
    f_n_c = 0
    v_p_c = 0
    v_n_c = 0
    
    for line in open(DATA_DIR + TEST_DATA_FILE, 'r'):
        if '[t]' in line:
            continue
        
        IAI = []
        split = line.split('##')
        sentence = split[1]
        if split[0] != '':
            for iai_aspect in split[0].split(','):
                IAI.append(iai_aspect.split('[')[0].strip(' '))
            
        tokens = bag_of_words(extract_words(sentence))
        imp = classifier.prob_classify(tokens).prob('imp')
        non = classifier.prob_classify(tokens).prob('non')
        
        print "imp: " + str(imp)
        print "non: " + str(non)
        
        if imp > 0.7:
            tagged_sentence = nltk.pos_tag((split[1].split(' ')))
            
            if IAI == []:
                f_p_c += 1
                for token in tagged_sentence:
                    if (token[0]  in PosWordList or token[0]  in NegWordList):
                        f_p += 1
                    else:
                        v_n += 1
            else:
                v_p_c += 1
                for token in tagged_sentence:
                    if token[0] in IAI:
                        if (token[0]  in PosWordList or token[0]  in NegWordList): 
                            v_p += 1
                        else:
                            f_n = f_n + 1
                    else:
                        if (token[0]  in PosWordList or token[0]  in NegWordList):
                            f_p = f_p + 1
                        else:
                            v_n = v_n + 1
        else:
            if IAI == []:
                v_n_c += 1
                v_n += 1
            else:
                f_n_c += 1
                f_n += 1
    
    print str(v_p)
    print str(f_p)
    print str(v_n)
    print str(f_n)
    
    P = float(float(v_p)/(float(v_p) + float(f_p)))
    R = float(float(v_p)/(float(v_p) + float(f_n)))

    print str(P)
    print str(R)
    
    F1 = float(2*(P*R)/(P+R))
    
    P_C = float(float(v_p_c)/(float(v_p_c) + float(f_p_c)))
    R_C = float(float(v_p_c)/(float(v_p_c) + float(f_n_c)))
    F1_C = float(2 * (P_C * R_C)/(P_C + R_C))

    
    print "Precision: " + str(P)
    print "Recall: " + str(R)
    print "F1: " + str(F1)  
                
    print "Precision Classifier: " + str(P_C)
    print "Recall Classifier: " + str(R_C)
    print "F1 Classifier: " + str(F1_C)                
        
