import glob
import string
import nltk

IA_RAW_DATA_DIR = '../Data/Implicit Aspects Annotated/'
EA_RAW_DATA_DIR = '../Data/Explicit Aspects Annotated/'

def IAIPOSProb():
    JJ_I = 0;
    JJ_O = 0;
            
    NN_I = 0;
    NN_O = 0;
            
    VB_I = 0;
    VB_O = 0;
    
    OT_I = 0;
    OT_O = 0;
    
    DocList = glob.glob(RAW_DATA_DIR + '*')
    for doc in DocList:
        f = open(doc, 'r')
        txt = f.readline()
        while txt != '':
            if '[t]' in txt:
                txt = f.readline()
                continue
            
            split = txt.split('##')
            if split[0] != '':
                sentence_wo_punct = split[1].translate(string.maketrans("",""), string.punctuation)
                sentence = nltk.word_tokenize(sentence_wo_punct)
                pos_tags = nltk.pos_tag(sentence)
                
                IAIList = [];
                for iai_aspect in split[0].split(','):
                    IAI = iai_aspect.split('[')[0]
                    IAI = IAI.strip(' ')
                    if IAI != '':
                        IAIList.append(IAI)
    
                for word_tag in pos_tags:
                    if 'JJ' in word_tag[1]:
                        if word_tag[0] in IAIList:
                            JJ_I += 1;
                        else:
                            JJ_O += 1;
                    elif 'NN' in word_tag[1]:
                        if word_tag[0] in IAIList:
                            NN_I += 1;
                        else:
                            NN_O += 1;
                    elif 'VB' in word_tag[1]:
                        if word_tag[0] in IAIList:
                            VB_I += 1;
                        else:
                            VB_O += 1;
                    else:
                        if word_tag[0] in IAIList:
                            OT_I += 1;
                        else:
                            OT_O += 1;

            
            txt = f.readline()
        f.close()
    
    print "JJ: " + str(JJ_I) + ' - ' + str(JJ_O) + " : " + str(float(JJ_I)/float(JJ_I + JJ_O))
    print "NN: " + str(NN_I) + ' - ' + str(NN_O) + " : " + str(float(NN_I)/float(NN_I + NN_O))
    print "VB: " + str(VB_I) + ' - ' + str(VB_O) + " : " + str(float(VB_I)/float(VB_I + VB_O))
    print "OT: " + str(OT_I) + ' - ' + str(OT_O) + " : " + str(float(OT_I)/float(OT_I + OT_O))
    return


def IACorpusStatistics():    
    DocList = glob.glob(IA_RAW_DATA_DIR + '*')
    
    IAI_JJ = 0
    IAI_NOT_JJ = 0
    
    
    for doc in DocList:
        SentencesNum = 0
        DocumentsNum = 0    
        ImplicitSentencesNum = 0
        Words = 0
        ImplicitWords = { }
        WordTypes = [] 
       
        f = open(doc, 'r')
        txt = f.readline()
        while txt != '':
            if '[t]' in txt:
                DocumentsNum += 1
                txt = f.readline()
                continue
            
            
            split = txt.split('##')
            if split[0] != '':
                ImplicitSentencesNum += 1
                
            sentence_wo_punct = split[1].translate(string.maketrans("",""), string.punctuation)
            sentence = nltk.word_tokenize(sentence_wo_punct)
            pos_tags = nltk.pos_tag(sentence)
            Words += len(sentence_wo_punct)
            
            for word in sentence:
                if word not in WordTypes:
                    WordTypes.append(word)
            
            for iai_aspect in split[0].split(','):
                IAI = iai_aspect.split('[')[0]
                IAI = IAI.strip(' ')
                if IAI != '':
                    if ImplicitWords.has_key(IAI):
                        ImplicitWords[IAI] += 1
                    else:
                        ImplicitWords[IAI] = 1
                    
                    l = [i for i, word in enumerate(pos_tags) if IAI in word[0]]

                    if l != []:
                        if 'NN' in pos_tags[l[0]][1] or 'VB' in pos_tags[l[0]][1] or 'JJ' in pos_tags[l[0]][1]: 
                            IAI_JJ += 1
                            #print "SI - " + pos_tags[l[0]][1]
                        else:
                            IAI_NOT_JJ += 1
                            #print "NO - " + pos_tags[l[0]][1]
            
            SentencesNum += 1
            txt = f.readline()
        f.close()
        
        #Total of implicit words
        TotalImplicitWords=0
        for key, value in ImplicitWords.iteritems():
            TotalImplicitWords += value
        
        print "-----------------------------------------------------------------"
        print doc
        print "Reviews: " + str(DocumentsNum)        
        print "Sentences: " + str(SentencesNum)
        print "Implicit Sentences: " + str(ImplicitSentencesNum)    
        print "Words: " + str(Words)
        print "Types: " + str(len(WordTypes))
        print "Words per review: " + str(float(Words)/float(DocumentsNum))
        print "Implicit Aspect Indicators Dictionary Lenght: " + str(len(ImplicitWords))
        print "Implicit Words: " + str(TotalImplicitWords)
        print "Sentences with Implicit Aspects: " + str(float(100.0 * ImplicitSentencesNum) / float(SentencesNum))
        print "-----------------------------------------------------------------"
    
    print "IAI_JJ: " + str(IAI_JJ)
    print "IAI_NOT_JJ: " + str(IAI_NOT_JJ)
    print "IAI JJ relation: " + str(100.0 * float(IAI_JJ) / float(IAI_JJ + IAI_NOT_JJ)) 
    return

def EACorpusStatistics():    
    DocList = glob.glob(EA_RAW_DATA_DIR + '*')
    
    
    for doc in DocList:
        SentencesNum = 0
        DocumentsNum = 0    
        ExplicitSentencesNum = 0
        Words = 0
        ExplicitWords = { }
       
        f = open(doc, 'r')
        txt = f.readline()
        while txt != '':
            if '[t]' in txt:
                DocumentsNum += 1
                txt = f.readline()
                continue
            
            
            split = txt.split('##')
            if split[0] != '':
                ExplicitSentencesNum += 1
            
            sentence_wo_punct = split[1].translate(string.maketrans("",""), string.punctuation)
            sentence = nltk.word_tokenize(sentence_wo_punct)
            pos_tags = nltk.pos_tag(sentence)
            Words += len(sentence_wo_punct)
            
            for e_token in split[0].split(','):
                ea = e_token.split('[')[0]
                ea = ea.strip(' ')
                if ea != '':
                    if ExplicitWords.has_key(ea):
                        ExplicitWords[ea] += 1
                    else:
                        ExplicitWords[ea] = 1
            
            SentencesNum += 1
            txt = f.readline()
        f.close()
 
        print "-----------------------------------------------------------------"
        print doc
        print "Reviews: " + str(DocumentsNum)        
        print "Sentences: " + str(SentencesNum)
        print "Implicit Sentences: " + str(ExplicitSentencesNum)    
        print "Words: " + str(Words)
        print "Words per review: " + str(float(Words)/float(DocumentsNum))
        print "Explicit Aspect Dictionary Lenght: " + str(len(ExplicitWords))
        print "Sentences with Explicit Aspects: " + str(float(100.0 * ExplicitSentencesNum) / float(SentencesNum))
        print "-----------------------------------------------------------------"

    return
                                                                                     
if __name__ == '__main__':
    IACorpusStatistics()
    